#include <stdio.h>
#include "bldc.h"

volatile long msec = 0;

int cur_led_color;
int cur_led_flashes;

static inline void
set_led_color (LED_Color c)
{
  switch (c)
    {
    case led_black:	RED = 0; GREEN = 0; BLUE = 0; break;
    case led_red:	RED = 1; GREEN = 0; BLUE = 0; break;
    case led_green:	RED = 0; GREEN = 1; BLUE = 0; break;
    case led_yellow:	RED = 1; GREEN = 1; BLUE = 0; break;
    case led_blue:	RED = 0; GREEN = 0; BLUE = 1; break;
    case led_magenta:	RED = 1; GREEN = 0; BLUE = 1; break;
    case led_cyan:	RED = 0; GREEN = 1; BLUE = 1; break;
    case led_white:	RED = 1; GREEN = 1; BLUE = 1; break;
    }
}

static void __attribute__((interrupt))
cmt0_handler ()
{
  msec ++;

  if ((msec & 127) == 0 && cur_led_flashes)
    {
      if (msec & 128)
	{
	  RED = 0;
	  GREEN = 0;
	  BLUE = 0;
	}
      else
	set_led_color (cur_led_color);
    }
}  

void
led_color (LED_Color c)
{
  cur_led_flashes = 0;
  set_led_color (c);
}

void
led_flash (LED_Color c)
{
  if (c == led_black)
    {
      cur_led_flashes = 0;
      set_led_color (c);
    }
  else
    {
      cur_led_color = c;
      cur_led_flashes = 1;
      set_led_color (c);
    }
}

void
bldc_init ()
{
  MSTP(RAM) = 0;
  MSTP(SCI1) = 0;
  MSTP(MTU) = 0;
  MSTP(CMT0) = 0;
  MSTP(S12AD0) = 0;
  MSTP(S12AD1) = 0;
  MSTP(CAN0) = 0;

  /* We start with a 10 MHz crystal.  */
  /* 0 = x8  1 = x4  2 = x2  3 = x1 */
  /* ICLK: x8 =  80 MHz  */
  /* PCLK: x4 =  40 MHz   */
  /* BCLK: x4 =  40 MHz   */
  /*                     I B P	*/
  SYSTEM.SCKCR.LONG = 0x00010100;
  /*	0 = 80 MHz
	1 = 40 MHz
	2 = 20 MHz
	3 = 10 MHz
  */
  
  setup_interrupts ();

  setbuf (stdout, NULL);
  setbuf (stderr, NULL);

  _vectors[VECT(CMT0,CMI0)] = cmt0_handler;
  IEN(CMT0,CMI0) = 1;
  IPR(CMT0,CMI0) = 5;

  CMT0.CMCOR = 40000 / 8;
  CMT0.CMCR.BIT.CKS = 0;  /* PCLK/8 @ 40 MHz */
  CMT0.CMCR.BIT.CMIE = 1; /* interrupts */
  CMT.CMSTR0.BIT.STR0 = 1; /* start */

  PORT7.DR.BYTE = UL | VL | WL;

  /* U/V/W out */
  PORT7.DDR.BYTE = UH | UL | VH | VL | WH | WL;
  /* Hall sensors */
  PORTB.ICR.BYTE = U_HALL | V_HALL | W_HALL;
  /* Encoders */
  PORT3.ICR.BYTE = 0x0f;
  PORTA.ICR.BYTE = 0x20;
  /* Fault */
  PORT7.ICR.BIT.B0 = 1;

  /* CAN-B */
  IOPORT.PFJCAN.BIT.CANE = 1;
  IOPORT.PFJCAN.BIT.CANS = 1;
  PORT2.DDR.BIT.B3 = 1;	//CTX0 is output
  PORT2.DDR.BIT.B2 = 0;	//CRX0 is input
  PORT2.ICR.BIT.B2 = 1;

  /* Limit switches */
  PORT9.DDR.BYTE = 0x00;
  PORT9.ICR.BYTE = 0x1e;

  /* Step/Dir interface */
  PORTA.DDR.BIT.B2 = 1; /* LIMIT2 */
  PORTA.DDR.BIT.B3 = 1; /* LIMIT1 */
  PORTB.DDR.BIT.B0 = 1; /* FAULT */
  /* PORTE.ICR.BIT.B2 = 1; */ /* E-STOP in - assumed */
  PORT2.DDR.BIT.B2 = 0; /* E-STOP out */
  PORT2.DR.BIT.B2 = 0;  /* E-STOP out */

  DRRED = 1;
  DRGREEN = 1;
  DRBLUE = 1;
  RED = 0;
  GREEN = 0;
  BLUE = 0;

  tty_init ();
  tty2_init ();
  thermistor_init ();
  encoder_init ();
  step_direction_init ();
  cpwm_init ();
  motor_init ();

  /* let the high-side caps charge */
  wait_msec (10);
}

void
bldc_exit ()
{
  tty_putc(3);
  tty_putc(3);
  tty_putc(0);
  while (1)
    ;
}

void
wait_msec (int ms)
{
  unsigned long now = msec;
  asm ("setpsw I");
  while ((long)(msec - now) < ms)
    asm("");
  //cprintf("  %w\r", CMT0.CMCNT);
}
