#include <stdio.h>
#include <math.h>
#include "bldc.h"

struct {
  int temp_c;
  double ohms;
} table[] = {
#include "thermistor-table.h"
};

#define MAX_TAB (sizeof(table) / sizeof(table[0]))

void
thermistor_init ()
{
  /* AN000 is thermistor.  Set it for 12-bit
     continuous conversion.  */
  S12AD0.ADANS.WORD = 0x1000;
  S12AD0.ADCER.BIT.ADRFMT = 1;
  S12AD0.ADCSR.BYTE = 0x4c;
  S12AD0.ADCSR.BYTE = 0xcc; /* start conversions */
}

#define BOTOHMS 6800.0

/* v = 4096 * (RU) / (Rt + RU)  */
/* (Rt + RU) v = 4096 * RU */
/* Rt * v + RU * v = 4096 * RU */
/* Rt * v = (4096 - v) * RU */
/* Rt = RU * (4096 - v) / v  */

static int last_idx = 0;

float
temperature ()
{
  int volts = THERM; /* 0..65536 */
  float ohms = (65536.0-volts) / volts * BOTOHMS;
  float temp_c, temp_f;

  while (1)
    {
      if (last_idx > 0 && ohms > table[last_idx].ohms)
	last_idx --;
      else if (last_idx < MAX_TAB-1 && ohms < table[last_idx+1].ohms)
	last_idx ++;
      else
	{
	  /* table[last_idx] is our best shot */
	  temp_c = table[last_idx].temp_c;
	  temp_c += (ohms - table[last_idx].ohms) / (table[last_idx+1].ohms - table[last_idx].ohms);
	  break;
	}
    }
  temp_f = temp_c * 9.0/5.0 + 32;

  /* printf ("%.1fC %.1fF", temp_c, temp_f);*/

  return temp_f;
}

float
motor_volts ()
{
  return VMOTOR / 256.0;
}
